/*
 *	File:		VUAE.h
 *
 *	Contains:	This file is part of the Virtual User 2.0 external tool interface.
 *
 *	Written by:	Automation Systems, Apple Computer, Inc.
 *
 *	Copyright:	 1992-1994 by Apple Computer, Inc., all rights reserved.
 *
 *	Change History (most recent first):
 *
 *		 <5>	12/13/94	CMW		Add errAEThreadsNotPresent enum.
 *		 <4+>	12/12/94	SBR		Add errAEThreadsNotPresent enum.
 *		 <4>	 12/3/94	CMW		Add kVUAEProtocolVersion parameter.
 *		 <3>	  7/7/94	CMW		Make required service strings of type 'const char* const'.
 *		 <2>	 1/13/94	CMW		Changed for Virtual User 2.1: kVUAESrvcResults = keyDirectObject
 *		 <9>	11/18/92	CMW		Add event id and wait amount parameter for dealing with
 *									timeouts. Removed unused parameter consts for GetToolVers.
 *		 <7>	11/13/92	RV		
 *		<6+>	11/13/92	RV		
 *		 <6>	11/12/92	CMW		Change occurances of "GetToolVers" to "GetToolVersion". Rename
 *									all constants starting with kAEVU to start with kVUAE.
 *		 <5>	 11/2/92	RV		Added Initialize string constant
 *		<4+>	 11/2/92	RV		Add "Initialize" string constant
 *		<2+>	 5/29/92	CMW		Change all occurances of "Command" to "Service" in the names of
 *									all the constants and events.
 *		<1+>	 3/11/92	CMW		Update to reflect the changes to the External Tool calls for
 *									2.0d3
 *		 <1>	 2/20/92	JCC		
 *		 <4>	 2/19/92	JCC		Finishing touches
 *		<3+>	 2/19/92	JCC		
 *		 <3>	 2/13/92	JCC		Rewriting the pascal commands in C
 *		<2+>	 2/13/92	JCC		Rewriting the pascal commands in C
 *		 <2>	 2/13/92	CMW		Add 'errAENoErr' to errors enumeration, convert #define'd
 *									constants to 'const's, and fix the font.
 *
 *	To Do:
*/


#ifndef __VUAE__
#define __VUAE__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

/* Standard constants used for communicating with VU External Tools */
const long kVUAETool			= 'v.u.';	/* event class for V.U. 2.0 external tool interface */
const long kVUAESendService		= 'extc';	/* event id for SendToolService */
const long kVUAEWaitLonger		= 'extw';	/* event id for WaitLongerForService */

const long kVUAESrvcName		= keyDirectObject;	/* Service name for external tool Service */
const long kVUAESrvcParameters	= 'extp';
const long kVUAESrvcResults		= keyDirectObject;
const long kVUAESrvcTarget		= 'targ';
const long kVUAEWaitAmount		= 'xtwa';
const long kVUAEProtocolVersion	= 'xtpv';	/* NEW FOR 2.1 */

const long kVUAEOldSrvcResults	= 'extr';	/* Do not use for future development: */
											/* (this is not AppleScript friendly) */

/* Errors specific to the VU External Tool Interface*/
enum VUAEErrors
{
	errAENoErr = 0,
	errAENeverReturnThisErrorCode = 1,	/* Never EVER return this number as an error code!!! */
	errAEValueTooSmall = 2,
	errAEWrongParameters = 3,			/* incorrect parameters passed to external tool */
	errAEUnknownService = 4,
	errAEThreadsNotPresent = 5			/* ThreadedService called, no threads present */
};

const char* const	kVUAEInitialize		= "Initialize";
const char* const	kVUAECancelService	= "Cancel";
const char* const	kVUAEGetToolVersion	= "GetToolVersion";
const char* const	kVUAEGetServiceList	= "GetToolServices";
const char* const	kVUAEHasService		= "ServiceSupported";
const char* const	kVUAEQuitService	= "Quit";

#endif
